@@ -1,17 +1,23 @@
 from __future__ import print_function
+
+import io
 import os
 import sys
 import unittest
-import io
+import warnings
 
 from xml.sax.saxutils import XMLGenerator
 from xml.sax import SAXParseException
 from pyexpat import ExpatError
 
-from defusedxml import cElementTree, ElementTree, minidom, pulldom, sax, xmlrpc
+from defusedxml import cElementTree, ElementTree, minidom, pulldom, sax, xmlrpc, expatreader
 from defusedxml import defuse_stdlib
-from defusedxml import (DTDForbidden, EntitiesForbidden,
-                        ExternalReferenceForbidden, NotSupportedError)
+from defusedxml import (
+    DTDForbidden,
+    EntitiesForbidden,
+    ExternalReferenceForbidden,
+    NotSupportedError,
+)
 from defusedxml.common import PY3
 
 
@@ -20,14 +26,25 @@ try:
 except ImportError:
     gzip = None
 
+
+if sys.version_info < (3, 7):
+    warnings.filterwarnings("once", category=DeprecationWarning)
+
+
 try:
-    from defusedxml import lxml
+    with warnings.catch_warnings(record=True) as lxml_warnings:
+        from defusedxml import lxml
     from lxml.etree import XMLSyntaxError
+
     LXML3 = lxml.LXML3
 except ImportError:
     lxml = None
     XMLSyntaxError = None
     LXML3 = False
+    lxml_warnings = None
+
+
+warnings.filterwarnings("error", category=DeprecationWarning, module=r"defusedxml\..*")
 
 
 HERE = os.path.dirname(os.path.abspath(__file__))
@@ -88,68 +105,57 @@ class BaseTests(DefusedTestCase):
         self.assertRaises(EntitiesForbidden, self.parse, self.xml_quadratic)
         self.assertRaises(EntitiesForbidden, self.parse, self.xml_external)
 
-        self.assertRaises(EntitiesForbidden, self.parseString,
-                          self.get_content(self.xml_bomb))
-        self.assertRaises(EntitiesForbidden, self.parseString,
-                          self.get_content(self.xml_quadratic))
-        self.assertRaises(EntitiesForbidden, self.parseString,
-                          self.get_content(self.xml_external))
+        self.assertRaises(EntitiesForbidden, self.parseString, self.get_content(self.xml_bomb))
+        self.assertRaises(
+            EntitiesForbidden, self.parseString, self.get_content(self.xml_quadratic)
+        )
+        self.assertRaises(
+            EntitiesForbidden, self.parseString, self.get_content(self.xml_external)
+        )
 
         if self.iterparse:
-            self.assertRaises(EntitiesForbidden, self.iterparse,
-                              self.xml_bomb)
-            self.assertRaises(EntitiesForbidden, self.iterparse,
-                              self.xml_quadratic)
-            self.assertRaises(EntitiesForbidden, self.iterparse,
-                              self.xml_external)
+            self.assertRaises(EntitiesForbidden, self.iterparse, self.xml_bomb)
+            self.assertRaises(EntitiesForbidden, self.iterparse, self.xml_quadratic)
+            self.assertRaises(EntitiesForbidden, self.iterparse, self.xml_external)
 
     def test_entity_cycle(self):
-        self.assertRaises(self.cyclic_error, self.parse, self.xml_cyclic,
-                          forbid_entities=False)
+        self.assertRaises(self.cyclic_error, self.parse, self.xml_cyclic, forbid_entities=False)
 
     def test_dtd_forbidden(self):
-        self.assertRaises(DTDForbidden, self.parse, self.xml_bomb,
-                          forbid_dtd=True)
-        self.assertRaises(DTDForbidden, self.parse, self.xml_quadratic,
-                          forbid_dtd=True)
-        self.assertRaises(DTDForbidden, self.parse, self.xml_external,
-                          forbid_dtd=True)
-        self.assertRaises(DTDForbidden, self.parse, self.xml_dtd,
-                          forbid_dtd=True)
+        self.assertRaises(DTDForbidden, self.parse, self.xml_bomb, forbid_dtd=True)
+        self.assertRaises(DTDForbidden, self.parse, self.xml_quadratic, forbid_dtd=True)
+        self.assertRaises(DTDForbidden, self.parse, self.xml_external, forbid_dtd=True)
+        self.assertRaises(DTDForbidden, self.parse, self.xml_dtd, forbid_dtd=True)
 
-        self.assertRaises(DTDForbidden, self.parseString,
-                          self.get_content(self.xml_bomb),
-                          forbid_dtd=True)
-        self.assertRaises(DTDForbidden, self.parseString,
-                          self.get_content(self.xml_quadratic),
-                          forbid_dtd=True)
-        self.assertRaises(DTDForbidden, self.parseString,
-                          self.get_content(self.xml_external),
-                          forbid_dtd=True)
-        self.assertRaises(DTDForbidden, self.parseString,
-                          self.get_content(self.xml_dtd),
-                          forbid_dtd=True)
+        self.assertRaises(
+            DTDForbidden, self.parseString, self.get_content(self.xml_bomb), forbid_dtd=True
+        )
+        self.assertRaises(
+            DTDForbidden, self.parseString, self.get_content(self.xml_quadratic), forbid_dtd=True
+        )
+        self.assertRaises(
+            DTDForbidden, self.parseString, self.get_content(self.xml_external), forbid_dtd=True
+        )
+        self.assertRaises(
+            DTDForbidden, self.parseString, self.get_content(self.xml_dtd), forbid_dtd=True
+        )
 
         if self.iterparse:
-            self.assertRaises(DTDForbidden, self.iterparse,
-                              self.xml_bomb, forbid_dtd=True)
-            self.assertRaises(DTDForbidden, self.iterparse,
-                              self.xml_quadratic, forbid_dtd=True)
-            self.assertRaises(DTDForbidden, self.iterparse,
-                              self.xml_external, forbid_dtd=True)
-            self.assertRaises(DTDForbidden, self.iterparse,
-                              self.xml_dtd, forbid_dtd=True)
+            self.assertRaises(DTDForbidden, self.iterparse, self.xml_bomb, forbid_dtd=True)
+            self.assertRaises(DTDForbidden, self.iterparse, self.xml_quadratic, forbid_dtd=True)
+            self.assertRaises(DTDForbidden, self.iterparse, self.xml_external, forbid_dtd=True)
+            self.assertRaises(DTDForbidden, self.iterparse, self.xml_dtd, forbid_dtd=True)
 
     def test_dtd_with_external_ref(self):
         if self.dtd_external_ref:
-            self.assertRaises(self.external_ref_exception, self.parse,
-                              self.xml_dtd)
+            self.assertRaises(self.external_ref_exception, self.parse, self.xml_dtd)
         else:
             self.parse(self.xml_dtd)
 
     def test_external_ref(self):
-        self.assertRaises(self.external_ref_exception, self.parse,
-                          self.xml_external, forbid_entities=False)
+        self.assertRaises(
+            self.external_ref_exception, self.parse, self.xml_external, forbid_entities=False
+        )
 
     def test_external_file_ref(self):
         content = self.get_content(self.xml_external_file)
@@ -158,13 +164,13 @@ class BaseTests(DefusedTestCase):
             content = content.replace(b"/PATH/TO", here)
         else:
             content = content.replace("/PATH/TO", HERE)
-        self.assertRaises(self.external_ref_exception, self.parseString,
-                          content, forbid_entities=False)
+        self.assertRaises(
+            self.external_ref_exception, self.parseString, content, forbid_entities=False
+        )
 
     def test_allow_expansion(self):
         self.parse(self.xml_bomb2, forbid_entities=False)
-        self.parseString(self.get_content(self.xml_bomb2),
-                         forbid_entities=False)
+        self.parseString(self.get_content(self.xml_bomb2), forbid_entities=False)
 
 
 class TestDefusedElementTree(BaseTests):
@@ -186,6 +192,18 @@ class TestDefusedElementTree(BaseTests):
     def iterparse(self, source, **kwargs):
         return list(self.module.iterparse(source, **kwargs))
 
+    def test_html_arg(self):
+        with self.assertRaises(DeprecationWarning):
+            ElementTree.XMLParse(html=0)
+        with self.assertRaises(TypeError):
+            ElementTree.XMLParse(html=1)
+
+    def test_aliases(self):
+        parser = self.module.DefusedXMLParser
+        assert self.module.XMLTreeBuilder is parser
+        assert self.module.XMLParser is parser
+        assert self.module.XMLParse is parser
+
 
 class TestDefusedcElementTree(TestDefusedElementTree):
     module = cElementTree
@@ -207,6 +225,21 @@ class TestDefusedMinidom(BaseTests):
         return doc.toxml()
 
 
+class TestDefusedMinidomWithParser(TestDefusedMinidom):
+    cyclic_error = SAXParseException
+    dtd_external_ref = True
+
+    def parse(self, xmlfile, **kwargs):
+        doc = self.module.parse(xmlfile, parser=expatreader.create_parser(**kwargs), **kwargs)
+        return doc.toxml()
+
+    def parseString(self, xmlstring, **kwargs):
+        doc = self.module.parseString(
+            xmlstring, parser=expatreader.create_parser(**kwargs), **kwargs
+        )
+        return doc.toxml()
+
+
 class TestDefusedPulldom(BaseTests):
     module = pulldom
 
@@ -258,8 +291,10 @@ class TestDefusedSax(BaseTests):
 
         with self.assertRaises(ExternalReferenceForbidden) as ctx:
             self.parse(self.xml_external, forbid_entities=False)
-        msg = ("ExternalReferenceForbidden"
-               "(system_id='http://www.w3schools.com/xml/note.xml', public_id=None)")
+        msg = (
+            "ExternalReferenceForbidden"
+            "(system_id='http://www.w3schools.com/xml/note.xml', public_id=None)"
+        )
         self.assertEqual(str(ctx.exception), msg)
         self.assertEqual(repr(ctx.exception), msg)
 
@@ -292,6 +327,7 @@ class TestDefusedLxml(BaseTests):
         return self.module.tostring(tree)
 
     if not LXML3:
+
         def test_entities_forbidden(self):
             self.assertRaises(NotSupportedError, self.parse, self.xml_bomb)
 
@@ -306,8 +342,7 @@ class TestDefusedLxml(BaseTests):
 
     def test_restricted_element1(self):
         try:
-            tree = self.module.parse(self.xml_bomb, forbid_dtd=False,
-                                     forbid_entities=False)
+            tree = self.module.parse(self.xml_bomb, forbid_dtd=False, forbid_entities=False)
         except XMLSyntaxError:
             self.skipTest("lxml detects entityt reference loop")
         root = tree.getroot()
@@ -324,8 +359,7 @@ class TestDefusedLxml(BaseTests):
 
     def test_restricted_element2(self):
         try:
-            tree = self.module.parse(self.xml_bomb2, forbid_dtd=False,
-                                     forbid_entities=False)
+            tree = self.module.parse(self.xml_bomb2, forbid_dtd=False, forbid_entities=False)
         except XMLSyntaxError:
             self.skipTest("lxml detects entityt reference loop")
         root = tree.getroot()
@@ -370,9 +404,13 @@ class TestDefusedLxml(BaseTests):
         self.assertEqual(len(elements), 1)
         self.assertEqual(elements, list(root)[:1])
 
+    def test_lxml_warnings(self):
+        self.assertTrue(lxml_warnings)
+        self.assertEqual(lxml_warnings[0].category, DeprecationWarning)
+        self.assertIn("tests.py", lxml_warnings[0].filename)
+
 
 class XmlRpcTarget(object):
-
     def __init__(self):
         self._data = []
 
@@ -415,8 +453,7 @@ class TestXmlRpc(DefusedTestCase):
         self.assertRaises(EntitiesForbidden, self.parse, self.xml_bomb)
         self.assertRaises(EntitiesForbidden, self.parse, self.xml_quadratic)
         self.parse(self.xml_dtd)
-        self.assertRaises(DTDForbidden, self.parse, self.xml_dtd,
-                          forbid_dtd=True)
+        self.assertRaises(DTDForbidden, self.parse, self.xml_dtd, forbid_dtd=True)
 
     # def test_xmlrpc_unpatched(self):
     #    for fname in (self.xml_external,  self.xml_dtd):
@@ -430,7 +467,6 @@ class TestXmlRpc(DefusedTestCase):
 
 
 class TestDefusedGzip(DefusedTestCase):
-
     def get_gzipped(self, length):
         f = io.BytesIO()
         gzf = gzip.GzipFile(mode="wb", fileobj=f)
@@ -487,6 +523,7 @@ def test_main():
     suite.addTests(unittest.makeSuite(TestDefusedcElementTree))
     suite.addTests(unittest.makeSuite(TestDefusedElementTree))
     suite.addTests(unittest.makeSuite(TestDefusedMinidom))
+    suite.addTests(unittest.makeSuite(TestDefusedMinidomWithParser))
     suite.addTests(unittest.makeSuite(TestDefusedPulldom))
     suite.addTests(unittest.makeSuite(TestDefusedSax))
     suite.addTests(unittest.makeSuite(TestXmlRpc))
